function crm33000000_Gerenciamento_De_AgendamentoController(ScreenService, CRM_Utilities, General, Messages, restEngine, templateManager) {

    /**
     * Função para cancelar o gerenciamento do assistente;
     * 
     * @param {object} widget widget de gerenciamento
     * do assistente.
    */
    this.cancelScheduling = function (widget) {

        var parentWidget = templateManager.container.getWidget('WDGT_ASSISTENTE_AGENDAMENTO');

        ScreenService.closeSwipe();
        parentWidget.activate();

    };

    /**
     * Função para confirmar se o usuário
     * realmente quer gerar as tarefas para
     * os parceiros filtrados na widget principal.
     * 
     * @param {object} widget widget responsável
     * pelo gerenciamento do assistente(WDGT_GERENCIAMENTO_ASSISTENETE).
    */
    this.confirmGenerator = function (widget) {
        var mainWidget = templateManager.container.getWidget('WDGT_ASSISTENTE_AGENDAMENTO');
        var numberOfPartners = this.getNumberOfPartnersToGenerateTasks(mainWidget);

        if (widget.isValid()) {
            ScreenService.confirmMessage(Messages.CONFIRMACAO_GERADOR_DE_TAREFAS(numberOfPartners), 'question', function (success) {
                this.generateTasks(widget);
            }.bind(this));
        }
    };

    /**
     * Retornar o número de parceiros configurado
     * no assistente.
     * 
     * @param {object} widget widget principal do
     * assistente.
    */
    this.getNumberOfPartnersToGenerateTasks = function (widget) {

        if (widget.hasCheckedRows()) {

            var selectedPartners = widget.getCheckedRows();
            return selectedPartners.length;

        } else {
            return widget.dataSource.data.length;
        }

    };

    /**
     * Função para gerar as tarefas para os
     * parceiros filtrados na widget principal.
     * 
     * @param {object} widget widget responsável
     * pelo gerenciamento do assistente(WDGT_GERENCIAMENTO_ASSISTENETE).
    */
    this.generateTasks = function (widget) {

        widget.currentRow.CRM_NRORGTRAB = getLocalVar('CRM_NRORGTRABALHO');
        var assistantMainWidget = templateManager.container.getWidget('WDGT_ASSISTENTE_AGENDAMENTO');

        this.createTasksCampaign(widget).then(function (response) {

            var operatorsDataSet = this.configureRowsForTheTaskGenerator(assistantMainWidget, widget);

            General.callRestEngine("product#/crm_candidato_prospect_infosparc/generateTasks", "DataSet", operatorsDataSet).then(
                function (response) {
                    if (response.dataset && response.dataset.error) {
                        ScreenService.showMessage(response.dataset.error, 'ERROR');
                    }
                    if (response.dataset && (response.dataset.msg || response.dataset.msg)) {
                        var dataset = response.dataset;

                        ScreenService.notificationMessage("Ocorreram erros na adição dos registros, verifique o popup de mensagens!", "alert");
                        openPopupMensagens(dataset.msg);
                    } else {
                        ScreenService.showMessage(Messages.TAREFAS_GERADAS_SUCESSO);
                    }

                    ScreenService.closeSwipe();
                    assistantMainWidget.activate();

                }.bind(this),
                function (error) {
                    ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error.data.error));
                }
            );

        }.bind(this));

    };

    /**
     * Salva a campanha referente as tarefas que
     * serão geradas.
     * 
     * @param {object} widget widget do gerenciamento
     * do assistente.
    */
    this.createTasksCampaign = function (widget) {

        var currentRow = widget.currentRow;

        var params = {
            "requestType": "Row",
            "row": currentRow
        };

        return restEngine.request("product#/crm_campanhatarefa/save", params).then(
            function (response) {
                widget.currentRow.NRSEQCAMPANHA = response.dataset.DATA.NRSEQCAMPANHA;
            }.bind(this),
            function (error) {

            }
        );

    };

    /**
     * Função para configurar os dados das tarefas
     * que serão inseridas.
     * 
     * @param {object} assistantMainWidget widget principal
     * onde foram filtrados os parceiros(WDGT_ASSISTENTE_AGENDAMENTO).
     * 
     * @param {object} widget widget responsável
     * pelo gerenciamento do assistente(WDGT_GERENCIAMENTO_ASSISTENETE).
     * 
     * @return {object} operatorsDataSet dataset com os objetos(tarefas)
     * configurados.
    */
    this.configureRowsForTheTaskGenerator = function (assistantMainWidget, widget) {

        this.setOperatorsTaskInitialDate(widget);

        var operatorsDataSet = this.prepareRowsToSave(assistantMainWidget, widget);

        return operatorsDataSet;

    };

    /**
     * Função para criar um array referente ao array
     * de operadores(já existente na currentRow), responsável
     * por controlar a data das tarefas referente a cada operado.
     * 
     * @param {object} widget widget responsável
     * pelo gerenciamento do assistente(WDGT_GERENCIAMENTO_ASSISTENETE).
    */
    this.setOperatorsTaskInitialDate = function (widget) {

        var currentRow = widget.currentRow;

        var operators = currentRow.CDOPERADOR;
        var initialDate = currentRow.DATAINICIALTAREFAS;

        currentRow.DATARERERENCIAOPER = [];

        operators.forEach(function (operator, i) {

            currentRow.DATARERERENCIAOPER.push(initialDate);

        });

    };

    /**
     * Função para configurar os dados de cada tarefa
     * a ser inserida.
     *
     * @param {object} assistantMainWidget widget principal
     * onde foram filtrados os parceiros(WDGT_ASSISTENTE_AGENDAMENTO).
     * 
     * @param {object} widget widget responsável
     * pelo gerenciamento do assistente(WDGT_GERENCIAMENTO_ASSISTENETE).
    */
    this.prepareRowsToSave = function (assistantMainWidget, widget) {

        var currentRow = widget.currentRow;

        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();

        var operators = currentRow.CDOPERADOR;
        var partners = this.getPartnersToGenerateTasks(assistantMainWidget);

        var numberOfTasksByOperator = currentRow.NRTAREFASPORDIA;

        var taskCount = 0;
        var referenceOperator = 0;

        var row;
        var operatorTaskDate;

        return partners.map(function (partner) {

            row = {};

            if (taskCount === numberOfTasksByOperator) {

                currentRow.DATARERERENCIAOPER[referenceOperator] = this.addWorkingDaysToDate(operatorTaskDate, 1);

                referenceOperator = (++referenceOperator) % operators.length;
                taskCount = 0;

            }

            operatorTaskDate = currentRow.DATARERERENCIAOPER[referenceOperator];

            row.__is_new = true;
            row.DSFILACONTATO = (currentRow.NMCAMPANHA) ? '[' + currentRow.NMCAMPANHA + '] ' + currentRow.DESCTAREFA : currentRow.DESCTAREFA;
            row.DTPLANFILACONTA = operatorTaskDate;
            row.IDORIGFILACONTA = 'M';
            row.NRPARCNEGOCIO = partner.NRPARCNEGOCIO;
            row.NRPRIORFILACONT = currentRow.NRPRIORIDADETAREFA;
            row.CDOPERREAL = operators[referenceOperator];
            row.NRORG = loggedOperatorInfos.NRORG;
            row.CRM_NRORGTRAB = getLocalVar('CRM_NRORGTRABALHO');
            row.NRSEQCONTATO = partner.NRSEQCONTATO;
            row.NRSEQCAMPANHA = currentRow.NRSEQCAMPANHA;

            taskCount++;

            return row;

        }.bind(this));
    };

    /**
     * Retorna os parceiros requerido pelo
     * assistente.
     * 
     * @param {object} widget widget principal
     * do assistente.
    */
    this.getPartnersToGenerateTasks = function (widget) {

        if (widget.hasCheckedRows()) {
            return widget.getCheckedRows();
        } else {
            return widget.dataSource.data;
        }

    };

    /**
     * Função para somar um número de dias úteis
     * a uma determinada data.
     * 
     * @param {string} date data que será alterada.
     * 
     * @param {number} numberOfWorkingDays número 
     * de dias úteis que serão acrescentados a data 
     * recebida com parâmetro.
    */
    this.addWorkingDaysToDate = function (date, numberOfWorkingDays) {

        var newDate = this.addDaysToADate(date, numberOfWorkingDays);

        switch (this.getDayOfTheWeekRelativeTheDate(newDate)) {

            case "Sat":
                newDate = this.addDaysToADate(newDate, 2);
                break;

            case "Sun":
                newDate = this.addDaysToADate(newDate, 1);
                break;
        }

        return newDate;

    };

    /**
     * Função para retornar o dia da semana(simplificado)
     * referente a data recebida como parâmetro.
     * 
     * @param {string} date date referência.
     * 
     * @return {string} nome simplificado do dia da semana
     * referente a date recebida como parâmetro
    */
    this.getDayOfTheWeekRelativeTheDate = function (date) {

        var dateMomentObject = moment(date, 'DD/MM/YYYY HH:mm');

        return dateMomentObject.toString().substr(0, 3);

    };

    /**
     * Função para somar um número de dias a uma
     * determinada data.
     * 
     * @param {string} date data que será alterada.
     * 
     * @param {number} numberOfDays número de dias
     * que serão acrescentados a data recebida com
     * parâmetro.
    */
    this.addDaysToADate = function (date, numberOfDays) {

        var newDate = moment(date, 'DD/MM/YYYY HH:mm').add(numberOfDays, 'days').format('DD/MM/YYYY HH:mm');

        return newDate;

    };

    var openPopupMensagens = function (msg) {
        var widgetMensagens = templateManager.container.getWidget('messagesAgrupamento');
        widgetMensagens.dataSource.data = msg;
        ScreenService.openPopup(widgetMensagens);
    };

}

Configuration(function (ContextRegister) {
    ContextRegister.register('crm33000000_Gerenciamento_De_AgendamentoController', crm33000000_Gerenciamento_De_AgendamentoController);
});